@@ -101,7 +101,7 @@ class MuminDataset:
               Network Posts_ (2021)
     '''
     download_url: str = ('https://github.com/CLARITI-REPHRAIN/mumin-build/raw/'
-                         'c6700f77c4c6cffcdb0bbfad21feb40c153f9a75'
+                         '3221d28beaff514bcdfc0eebaefa50735810f9bd'
                          '/data/mumin.zip')
     _node_dump: List[str] = ['claim', 'tweet', 'user', 'image', 'article',
                              'hashtag', 'reply']
@@ -376,13 +376,21 @@ class MuminDataset:
 
             # Filter (:Tweet)-[:DISCUSSES]->(:Claim)
             discusses_rel = (self.rels[('tweet', 'discusses', 'claim')]
-                             .query(f'relevance > {threshold}')
+                             .query('relevance > @threshold')
                              .reset_index(drop=True))
             self.rels[('tweet', 'discusses', 'claim')] = discusses_rel
 
             # Filter tweets
+            src_tweets = discusses_rel.src.unique().tolist()
+            user2tweet = self.rels[('user', 'posted', 'tweet')]
+            users = (user2tweet[user2tweet.tgt.isin(src_tweets)].src
+                                                                .unique()
+                                                                .tolist())
+            tweets = (user2tweet[user2tweet.src.isin(users)].tgt
+                                                            .unique()
+                                                            .tolist())
             tweet_df = self.nodes['tweet']
-            include_tweet = tweet_df.tweet_id.isin(discusses_rel.src.tolist())
+            include_tweet = tweet_df.tweet_id.isin(tweets)
             tweet_df = tweet_df[include_tweet].reset_index(drop=True)
             self.nodes['tweet'] = tweet_df
 
@@ -498,23 +506,44 @@ class MuminDataset:
         Args:
             node_type (str): The type of node to rehydrate.
         '''
-
         if (node_type in self.nodes.keys() and
                 (node_type != 'reply' or self.include_replies)):
 
             logger.info(f'Rehydrating {node_type} nodes')
 
-            # Get the tweet IDs
-            tweet_ids = self.nodes[node_type].tweet_id.tolist()
+            # Get the tweet IDs, and if the node type is a tweet then separate
+            # these into source tweets and the rest (i.e., timeline tweets)
+            if node_type == 'tweet':
+                source_tweet_ids = (self.rels[('tweet', 'discusses', 'claim')]
+                                    .src
+                                    .tolist())
+                tweet_ids = [tweet_id
+                             for tweet_id in self.nodes[node_type]
+                                                 .tweet_id
+                                                 .tolist()
+                             if tweet_id not in source_tweet_ids]
+            else:
+                source_tweet_ids = list()
+                tweet_ids = self.nodes[node_type].tweet_id.tolist()
 
             # Store any features the nodes might have had before hydration
             prehydration_df = self.nodes[node_type].copy()
 
             # Rehydrate the tweets
-            tweet_dfs = self._twitter.rehydrate_tweets(tweet_ids=tweet_ids)
+            if node_type == 'tweet':
+                src_dict = dict(tweet_ids=source_tweet_ids)
+                tweet_dfs = self._twitter.rehydrate_tweets(tweet_ids=tweet_ids)
+                source_tweet_dfs = self._twitter.rehydrate_tweets(**src_dict)
+            else:
+                tweet_dfs = self._twitter.rehydrate_tweets(tweet_ids=tweet_ids)
+                source_tweet_dfs = {key: pd.DataFrame()
+                                    for key in tweet_dfs.keys()}
 
             # Extract and store tweets and users
-            self.nodes[node_type] = (tweet_dfs['tweets']
+            tweet_df = pd.concat([source_tweet_dfs['tweets'],
+                                  tweet_dfs['tweets']],
+                                 ignore_index=True)
+            self.nodes[node_type] = (tweet_df
                                      .drop_duplicates(subset='tweet_id')
                                      .reset_index(drop=True))
             if ('user' in self.nodes.keys() and
@@ -537,8 +566,11 @@ class MuminDataset:
             # Note: This will store `self.nodes['image']`, but this is only
             #       to enable extraction of URLs later on. The
             #       `self.nodes['image']` will be overwritten later on.
-            if self.include_tweet_images and len(tweet_dfs['media']):
-                image_df = (tweet_dfs['media']
+            if (node_type == 'tweet' and
+                    self.include_tweet_images and
+                    len(tweet_dfs['media'])):
+
+                image_df = (source_tweet_dfs['media']
                             .query('type == "photo"')
                             .drop_duplicates(subset='media_key')
                             .reset_index(drop=True))
@@ -729,6 +761,18 @@ class MuminDataset:
         '''Extracts nodes from the raw Twitter data'''
         logger.info('Extracting nodes')
 
+        # Define dataframe with the source tweets
+        discusses_rel = self.rels[('tweet', 'discusses', 'claim')]
+        is_src = self.nodes['tweet'].index.isin(discusses_rel.src.tolist())
+        src_tweets = self.nodes['tweet'][is_src]
+
+        # Define dataframe with the source users
+        posted_rel = self.rels[('user', 'posted', 'tweet')]
+        is_src = (posted_rel[posted_rel.tgt.isin(src_tweets.index.tolist())]
+                  .src
+                  .tolist())
+        src_users = self.nodes['user'][self.nodes['user'].index.isin(is_src)]
+
         # Hashtags
         if self.include_hashtags:
             def extract_hashtag(dcts: List[dict]) -> List[str]:
@@ -742,13 +786,13 @@ class MuminDataset:
                 '''
                 return [dct.get('tag') for dct in dcts]
 
-            # Add hashtags from tweets
+            # Add hashtags from source tweets
             if 'entities.hashtags' in self.nodes['tweet'].columns:
-                hashtags = (self.nodes['tweet']['entities.hashtags']
-                                .dropna()
-                                .map(extract_hashtag)
-                                .explode()
-                                .tolist())
+                hashtags = (src_tweets['entities.hashtags']
+                            .dropna()
+                            .map(extract_hashtag)
+                            .explode()
+                            .tolist())
                 node_df = pd.DataFrame(dict(tag=hashtags))
                 if 'hashtag' in self.nodes.keys():
                     node_df = (self.nodes['hashtag']
@@ -757,13 +801,13 @@ class MuminDataset:
                                    .reset_index(drop=True))
                 self.nodes['hashtag'] = node_df
 
-            # Add hashtags from users
+            # Add hashtags from source users
             if 'entities.description.hashtags' in self.nodes['user'].columns:
-                hashtags = (self.nodes['user']['entities.description.hashtags']
-                                .dropna()
-                                .map(extract_hashtag)
-                                .explode()
-                                .tolist())
+                hashtags = (src_users['entities.description.hashtags']
+                            .dropna()
+                            .map(extract_hashtag)
+                            .explode()
+                            .tolist())
                 node_df = pd.DataFrame(dict(tag=hashtags))
                 if 'hashtag' in self.nodes.keys():
                     node_df = (self.nodes['hashtag']
@@ -772,7 +816,7 @@ class MuminDataset:
                                    .reset_index(drop=True))
                 self.nodes['hashtag'] = node_df
 
-        # Add urls from tweets
+        # Add urls from source tweets
         if 'entities.urls' in self.nodes['tweet'].columns:
             def extract_url(dcts: List[dict]) -> List[Union[str, None]]:
                 '''Extracts urls from a list of dictionaries.
@@ -785,11 +829,11 @@ class MuminDataset:
                 '''
                 return [dct.get('expanded_url') or dct.get('url')
                         for dct in dcts]
-            urls = (self.nodes['tweet']['entities.urls']
-                        .dropna()
-                        .map(extract_url)
-                        .explode()
-                        .tolist())
+            urls = (src_tweets['entities.urls']
+                    .dropna()
+                    .map(extract_url)
+                    .explode()
+                    .tolist())
             node_df = pd.DataFrame(dict(url=urls))
             if 'url' in self.nodes.keys():
                 node_df = (self.nodes['url']
@@ -801,14 +845,14 @@ class MuminDataset:
         # Add urls from images
         if (self.include_tweet_images and
                 'attachments.media_keys' in self.nodes['tweet'].columns):
-            urls = (self.nodes['tweet'][['attachments.media_keys']]
-                        .dropna()
-                        .explode('attachments.media_keys')
-                        .merge(self.nodes['image'][['media_key', 'url']],
-                               left_on='attachments.media_keys',
-                               right_on='media_key')
-                        .url
-                        .tolist())
+            urls = (src_tweets[['attachments.media_keys']]
+                    .dropna()
+                    .explode('attachments.media_keys')
+                    .merge(self.nodes['image'][['media_key', 'url']],
+                           left_on='attachments.media_keys',
+                           right_on='media_key')
+                    .url
+                    .tolist())
             node_df = pd.DataFrame(dict(url=urls))
             if 'url' in self.nodes.keys():
                 node_df = (self.nodes['url']
@@ -817,7 +861,7 @@ class MuminDataset:
                                .reset_index(drop=True))
             self.nodes['url'] = node_df
 
-        # Add urls from user urls
+        # Add urls from source user urls
         if 'entities.url.urls' in self.nodes['user'].columns:
             def extract_url(dcts: List[dict]) -> List[Union[str, None]]:
                 '''Extracts urls from a list of dictionaries.
@@ -830,11 +874,11 @@ class MuminDataset:
                 '''
                 return [dct.get('expanded_url') or dct.get('url')
                         for dct in dcts]
-            urls = (self.nodes['user']['entities.url.urls']
-                        .dropna()
-                        .map(extract_url)
-                        .explode()
-                        .tolist())
+            urls = (src_users['entities.url.urls']
+                    .dropna()
+                    .map(extract_url)
+                    .explode()
+                    .tolist())
             node_df = pd.DataFrame(dict(url=urls))
             if 'url' in self.nodes.keys():
                 node_df = (self.nodes['url']
@@ -843,7 +887,7 @@ class MuminDataset:
                                .reset_index(drop=True))
             self.nodes['url'] = node_df
 
-        # Add urls from user descriptions
+        # Add urls from source user descriptions
         if 'entities.description.urls' in self.nodes['user'].columns:
             def extract_url(dcts: List[dict]) -> List[Union[str, None]]:
                 '''Extracts urls from a list of dictionaries.
@@ -856,11 +900,11 @@ class MuminDataset:
                 '''
                 return [dct.get('expanded_url') or dct.get('url')
                         for dct in dcts]
-            urls = (self.nodes['user']['entities.description.urls']
-                        .dropna()
-                        .map(extract_url)
-                        .explode()
-                        .tolist())
+            urls = (src_users['entities.description.urls']
+                    .dropna()
+                    .map(extract_url)
+                    .explode()
+                    .tolist())
             node_df = pd.DataFrame(dict(url=urls))
             if 'url' in self.nodes.keys():
                 node_df = (self.nodes['url']
@@ -883,9 +927,9 @@ class MuminDataset:
                 '''
                 return [dct.get('expanded_url') or dct.get('url')
                         for dct in dcts]
-            urls = (self.nodes['user']['profile_image_url']
-                        .dropna()
-                        .tolist())
+            urls = (src_users['profile_image_url']
+                    .dropna()
+                    .tolist())
             node_df = pd.DataFrame(dict(url=urls))
             if 'url' in self.nodes.keys():
                 node_df = (self.nodes['url']
